#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <openssl/bio.h>
#include <openssl/e_os2.h>
#include <openssl/buffer.h>
#include <openssl/crypto.h>

int main(int argc, char *argv[])
{
	// write to BIO memory
	BIO *mem, *bio_out;
	char data[] = "BIO test ...\n";
	mem = BIO_new(BIO_s_mem());
	if (mem == NULL) {
		printf("Error using BIO_new(BIO_s_mem(void))");
		return 1;
	}
	BIO_puts(mem, "BIO test ...\n");
	bio_out = BIO_new_fp(stdout, BIO_NOCLOSE);
	if (bio_out == NULL) {
		printf("Error using BIO_new_fp()");
		return 1;
	}
 	BIO_write(bio_out, data, strlen(data));
	BIO_free_all(bio_out);
	BIO_free(mem);

	// create & write to BIO file
	BIO *out_file;
	out_file = BIO_new(BIO_s_file());
	if (out_file == NULL) {
		printf("Error using BIO_new(BIO_s_file(void))");
		return 1;
	}
	if (!BIO_write_filename(out_file, "biotest.txt")) {
		printf("Error using BIO_write_filename(BIO*, string)");
		return 1;
	}
	BIO_printf(out_file, "BIO test ...\n");
	BIO_free(out_file);	
	
	// base64 encode
	BIO *bio, *b64;
	char message[] = "BIO test ...\n";
	printf("Encode text: %s", message);
 	b64 = BIO_new(BIO_f_base64());
	if (b64 == NULL) {
		printf("Error using BIO_new(BIO_f_base64(void))");
		return 1;
	}
 	bio = BIO_new_fp(stdout, BIO_NOCLOSE);
	if (bio == NULL) {
		printf("Error using BIO_new_fp()");
		return 1;
	}
 	bio = BIO_push(b64, bio);
	printf("Encoded text: ");
 	BIO_write(bio, message, strlen(message));
	BIO_flush(bio);
 	BIO_free_all(bio);

	return(0);
}
