// errtest.c
// check some error codes

#include <openssl/opensslconf.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <openssl/bio.h>
#include <openssl/err.h>
#include <openssl/rsa.h>

int main(int argc, char **argv)
{
	RSA *rsa;
	BIO *out;

	unsigned long err_bio, err_rsa;
	int err_lib, err_func, err_reason;
	const char *lib_str, *func_str, *reason_str;
	char *str_rsa;
	

	printf("warning: 'out' is used uninitialized in this function\n");
	BIO_set_fp(out, stdout, BIO_NOCLOSE|BIO_FP_TEXT); 

	err_bio = ERR_peek_error();
	printf("%s\n", ERR_error_string(err_bio, NULL)); 
	lib_str = ERR_lib_error_string(err_bio);
	func_str = ERR_func_error_string(err_bio);
	reason_str = ERR_reason_error_string(err_bio);					
	
	// if NULL => none registered error code
	printf("lib_str: %s\n", lib_str);
	printf("func_str: %s\n", func_str);
	printf("reason_str: %s\n", reason_str);

	printf("warning: 'rsa' is used uninitialized in this function\n");
	if (rsa == NULL)
	{
		while ((err_rsa = ERR_get_error()) != 0 && 
		       (str_rsa = ERR_error_string(err_rsa, NULL)) &&
		       (err_lib = ERR_GET_LIB(err_rsa)) != 0 && 
		       (err_func = ERR_GET_FUNC(err_rsa)) != 0 && 
                       (err_reason = ERR_GET_REASON(err_rsa)) != 0)
		{	
			printf("%s\n", str_rsa);
			printf("err_lib: %d\n", err_lib);
			printf("err_func: %d\n", err_func);
			printf("err_reason: %d\n", err_reason);
		}
	}

	return 0;
}
