// pemtest.c
// check some pem functionality and other

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <openssl/err.h>
#include <openssl/objects.h>
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/pkcs7.h>
#include <openssl/pem.h>

#undef PROG
#define PROG	pemtest_main

int main(int argc, char **argv)
{
	PKCS7 *p7 = NULL;
	BIO *in = NULL, *out = NULL, *bio_err = NULL;
	int i, badops = 0, text = 0;
	char *infile = NULL, *outfile = NULL, *prog;
	STACK_OF(X509) *certs = NULL;
	STACK_OF(X509_CRL) *crls = NULL;

	if (bio_err == NULL)
		if ((bio_err = BIO_new(BIO_s_file())) != NULL)
			BIO_set_fp(bio_err, stderr, BIO_NOCLOSE|BIO_FP_TEXT);

	prog = argv[0];
	argc--;
	argv++;
	while (argc >= 1)
	{
		if (strcmp(*argv, "-in") == 0)
		{
			if (--argc < 1) goto bad;
			infile = *(++argv);
		}
		else if (strcmp(*argv,"-out") == 0)
		{
			if (--argc < 1) goto bad;
			outfile = *(++argv);
		}
		else if (strcmp(*argv, "-text") == 0)
			text = 1;
		else
		{
			BIO_printf(bio_err, "unknown option %s\n", *argv);
			badops = 1;
			break;
		}
		argc--;
		argv++;
	}

	if (badops)
	{
bad:
		BIO_printf(bio_err, "%s [options]\n", prog);
		BIO_printf(bio_err, "where options are\n");
		BIO_printf(bio_err, " -in arg       input PKCS#7 file\n");
		BIO_printf(bio_err, " -out arg      output file\n");
		BIO_printf(bio_err, " -text         print full details of certificates\n");
		goto end;
	}

	printf("test begin's ...\n");
	ERR_load_crypto_strings();

	in = BIO_new(BIO_s_file());
	out = BIO_new(BIO_s_file());
	if ((in == NULL) || (out == NULL))
	{
		ERR_print_errors(bio_err);
                goto end;
        }

	if (infile == NULL)
		BIO_set_fp(in, stdin, BIO_NOCLOSE);
	else
	{
		if (BIO_read_filename(in, infile) <= 0)
		if (in == NULL)
		{
			perror(infile);
			goto end;
		}
	}

	printf("read input file ...\n");
	// read input file
	p7 = PEM_read_bio_PKCS7(in, NULL, NULL, NULL);
	if (p7 == NULL)
	{
		BIO_printf(bio_err, "unable to load PKCS7 object\n");
		ERR_print_errors(bio_err);
		goto end;
	}
	
	if (outfile == NULL)
		BIO_set_fp(out,stdout,BIO_NOCLOSE);
	else
	{
		if (BIO_write_filename(out,outfile) <= 0)
		{
			perror(outfile);
			goto end;
		}
	}

	printf("print certs and/or crl of the input file ...\n");		
	// print any certs and/or crl in the input file
	i = OBJ_obj2nid(p7->type);
	switch (i)
	{
		case NID_pkcs7_signed:
			certs = p7->d.sign->cert;
			crls = p7->d.sign->crl;
			break;
		case NID_pkcs7_signedAndEnveloped:
			certs = p7->d.signed_and_enveloped->cert;
			crls = p7->d.signed_and_enveloped->crl;
			break;
		default:
			break;
	}

	if (certs != NULL)
	{
		printf("certs:\n");
		X509 *x;

		for (i = 0; i < sk_X509_num(certs); i++)
		{
			x = sk_X509_value(certs, i);
			if (text) X509_print(out, x);

			PEM_write_bio_X509(out, x);
			BIO_puts(out, "\n");
		}
	}
	
	if (crls != NULL)
	{
		printf("crls:\n");
		X509_CRL *crl;

		for (i = 0; i < sk_X509_CRL_num(crls); i++)
		{
			crl = sk_X509_CRL_value(crls, i);
			X509_CRL_print(out, crl);
			PEM_write_bio_X509_CRL(out, crl);
			BIO_puts(out, "\n");
		}
	}

	printf("test end's ...\n");
	return 0;
end:
	if (p7 != NULL) PKCS7_free(p7);
	if (in != NULL) BIO_free(in);
	if (out != NULL) BIO_free_all(out);
	if (bio_err != NULL) BIO_free_all(bio_err);
	return 0;
}
