// txtdbtest.c

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <openssl/txt_db.h>
#include <openssl/bio.h>
#include <openssl/err.h>

#define DB_NUMBER	1	// field, defined in apps/apps.h

int main(int argc, char **argv)
{
	TXT_DB *db;
	BIO *bio_err = NULL, *in = NULL, *out = NULL;
	char *indbfile = NULL, *outdbfile = NULL, *prog;
	int badops = 0, dbwrite;
	
	if (bio_err == NULL)
		if ((bio_err = BIO_new(BIO_s_file())) != NULL)
			BIO_set_fp(bio_err, stderr, BIO_NOCLOSE|BIO_FP_TEXT);	

	prog = argv[0];
	argc--;
	argv++;
	while (argc >= 1)
	{
		if (strcmp(*argv, "-in") == 0)
		{
			if (--argc < 1) goto bad;
			indbfile = *(++argv);
		}
		else if (strcmp(*argv,"-out") == 0)
		{
			if (--argc < 1) goto bad;
			outdbfile = *(++argv);
		}
		argc--;
		argv++;
	}

	if (badops)
	{
bad:
		BIO_printf(bio_err, "%s [options]\n", prog);
		BIO_printf(bio_err, "where options are\n");
		BIO_printf(bio_err, " -in arg       input TXT_DB file\n");
		BIO_printf(bio_err, " -out arg      output file\n");
		goto end;
	}
	
	printf("test started ...\n");
	in = BIO_new(BIO_s_file());
	out = BIO_new(BIO_s_file());
	if ((in == NULL) || (out == NULL))
	{
		ERR_print_errors(bio_err);
		goto end;
	}

	// writing input file to bio_in
	if (BIO_read_filename(in, indbfile) <= 0)
	{
		perror(indbfile);
		BIO_printf(bio_err, "unable to open '%s'\n", indbfile);
		goto end;
	}

	// read bio_in file to TXT_DB
	printf("read input file to TXT_DB ...\n");
	if ((db = TXT_DB_read(in, DB_NUMBER)) == NULL)
		goto end;
	
	if (outdbfile == NULL)
		BIO_set_fp(out, stdout, BIO_NOCLOSE);
	else
	{
		if (BIO_write_filename(out, outdbfile) <= 0)
		{
			perror(outdbfile);
			goto end;
		}
	}	

	// writing TXT_DB to bio_out file
	printf("write TXT_DB to output file ...\n");
	dbwrite = TXT_DB_write(out, db);
	if (dbwrite <= 0) goto end;

	printf("test completed\n");
	return 0;
end:
	if (db) TXT_DB_free(db);
	if (in != NULL) BIO_free(in);
	if (out != NULL) BIO_free(out);
	if (bio_err != NULL) BIO_free_all(bio_err);
	return 0;
}
